#' Computes an approximation of the 2-Wasserstein-Barycenter of discrete input measures.
#' @description This function computes iteratively the best approximation of the data's Wasserstein-Barycenter
#' supported on a support of a specified size.
#' @param data.weights A list of vectors specifying the weights of the data measures. Each vector is assumed to
#' sum to one.
#' @param data.pos A list of Mxd matrices, specifying the positions of the data measures. Different measures do
#' not have to have the same support size and do not need to be supported on some grid.
#' @param sup_size A positive integer specifying the size of the support of the approximated barycenter.
#' @param maxiter A positive integer specifying the maximum number of iterations in the algorithm.
#' @param sub_iter A positive integer specifying the number of iterations in the weight-updates.
#' @param theta A real value in [0,1] used as a tuning parameter for the gradient descent.
#' @return The function returns a list with two entries. The first entry contains the positions of the computed
#' barycenter and the second entry contains the corresponding weights.
#' @examples res<-wsbary_cont(mnist_zero[[2]],mnist_zero[[1]],100,30,2)
#' @references Marco Cuturi and Arnaud Doucet; Fast computation of wasserstein barycenters;
#' In Proceedings of the 31st International Conference on International Conference on Machine Learning - Volume 32 (ICML'14); 2014.
#' @export
wsbary_cont<-function(data.weights,data.pos,sup_size,maxiter=50,sub_iter=5,theta=0.5){
  d<-dim(data.pos[[1]])[2]
  M<-length(data.weights)
  t0<-1
  gridsize<-ceiling(sup_size^(1/d))
  support<-build.grid_multidim(rep(gridsize,d))
  support<-support[1:sup_size,]
  pos.t<-lapply(data.pos,t)
  iter<-1
  sparse<-FALSE
  if (sup_size>6000){
    sparse<-TRUE
  }
  data.sizes<-lapply(data.weights,length)
  a.tilde<-rep(1/(sup_size),sup_size)
  a.hat<-a.tilde
  data.wpp<-mapply(wpp,data.pos,data.weights,SIMPLIFY = FALSE)
  while((iter<=maxiter)){
    tic<-proc.time()
    #dual weight update
    sub_count<-1
    while (sub_count<=sub_iter){
      beta<-(iter+1)/2
      a<-(1-beta^(-1))*a.hat+beta^(-1)*a.tilde
      a.wpp<-wpp(support,a)
      C<-lapply(pos.t,cost,A=t(support))
      duals<-mapply(solution_nongrid,data.wpp,C,MoreArgs = list(a=a.wpp,sparse=sparse),SIMPLIFY = FALSE)
      duals<-lapply(duals,normalise_dual,sup_size)
      alpha<-Reduce('+',duals)/M
      a.tilde<-a.tilde*exp((-1)*beta*alpha*t0)
      a.tilde<-a.tilde/sum(a.tilde)
      a.hat<-(1-beta^(-1))*a.hat+beta^(-1)*a.tilde
      sub_count<-sub_count+1
      gc()
    }
    a<-(1-beta^(-1))*a.hat+beta^(-1)*a.tilde
    a.wpp<-wpp(support,a)
    #primal position update
    plans<-lapply(data.wpp,transport,a=a.wpp,p=2)
    plans<-mapply(tp.create,plans,data.sizes,MoreArgs = list(d1=sup_size),SIMPLIFY = FALSE)
    up_step<-mapply(update_pos,data.pos,plans,SIMPLIFY = FALSE)
    up_step<-Reduce('+',up_step)/M
    up_step<-theta*((diag(a^(-1)))%*%up_step)
    support<-(1-theta)*support+up_step
    gc()
    #it
    iter<-iter+1
    #print(proc.time()-tic)
  }
  a<-(1-beta^(-1))*a.hat+beta^(-1)*a.tilde
  return(list(pos=support,weights=a))
}
