#' Creates a visualization of a generalized geodesic in the 2-Wasserstein-Space.
#' @description Creates a visualization of a general geodesic in the 2-Wasserstein-Space. It is parametrized as Y-V1+t(V1+V2), where Y is the matrix of positions of the
#' barycenter and V1 and V2 are velocity fields. It creates either a series of images or a gif to visualize the geodesic. The creation of a gif requieres the installation of
#' Imagemagik and the animation package.
#' @param bary.pos Nx2 Matrix of Positions of the barycenter in two dimensional space.
#' @param bary.weights A vector of the weights of each position of the barycenter
#' @param V1 A Nx2 matrix specifying the first velocity field
#' @param V2 A Nx2 matrix specifying the second velocity field
#' @param timesteps The amount of time steps between t=0 and t=1 at which the Geodesic is computed.
#' @param H The 2x2 bandwidth matrix for the computation of the kernel density estimator.
#' @param grid.size The size of the two dimensional grid on which the kde is evaluated.
#' @param create.gif Boolean determining whether the results should be exported as a gif. Note that this requieres the installation of the animation package and imagemagik.
#' @param create.images Boolean determining whether the results should be exported as a series of png-images.
#' @param file.name String determining the name of the exported files.
#' @param delay.time The time between two frames in the generated gif.
#' @return The function returns an array of the size gridsize x gridsize x (2+timesteps) containing the evaluation of the kde on the grid at each timepoint.
#' If create.image=TRUE, then the function exports the results as a series of png images at the corresponding time steps. If create.gif=TRUE, then it creates a gif
#' from this images to visualize the transport corresponding to the geodesic. Note that create.gif and create.images operate independent from each other, i.e. you
#' need to set both values to TRUE if you want both outputs.
#'@examples components<-pca_log(mnist_zero[[2]],mnist_zero[[1]],mnist_zero_bary[[2]],mnist_zero_bary[[1]])
#' for (c in 1:3){
#'  V<-components$components[[c]]*sqrt(components$evalues[c])
#'  name<-paste("RtestPCA","_pcanr_",c,sep="")
#'  visual_geodesic(mnist_zero_bary[[1]],mnist_zero_bary[[2]],V,V,timesteps = 100,grid.size=28*4,
#'  create.gif = FALSE,file.name=name)
#' }
#' @export
visual_geodesic<-function(bary.pos,bary.weights,V1,V2,timesteps=100,
                                     H = (1/1000)*matrix(c(1, 0, 0, 1), 2, 2),grid.size=100, create.gif=TRUE,create.images=TRUE, file.name = "RGeodesic", delay.time = 20){
  tic<-proc.time()
  if (!requireNamespace("ks", quietly = TRUE)) {
    stop("Package 'ks' required for computation of kernel density estimator. Please install it.")
  }
  data.dim<-length(bary.weights)
  geodesic<-array(0,c(grid.size,grid.size,timesteps+2))
  grid<-seq(0,1,1/(grid.size-1))
  v.temp <- rep(0, grid.size*grid.size)
  for (k in 1:grid.size) {
    v.temp[((k - 1) * grid.size + 1):(grid.size * k)] <- rep(k, grid.size)
  }
  grid.m <- cbind(v.temp,rep(grid.size:1, grid.size))
  grid.m <- (grid.m - 0.5)/grid.size
  for (k in 0:(timesteps+1)){
    t<-k/(timesteps+1)
    coord<-G(t,bary.pos,V1,V2)
    bary.weights<-bary.weights/sum(bary.weights)*length(bary.weights)
    geo.bin<-ks::kde(coord,w=bary.weights,eval.points=grid.m,H = H)
    geo.mat<-matrix(geo.bin$estimate, grid.size,grid.size)
    geo.mat<-geo.mat/sum(geo.mat)
    geodesic[,,(k+1)]<-geo.mat
  }
  toc<-proc.time()
  if (create.images==TRUE){
    for (k in 1:(timesteps+2)){
      image.name<-paste(file.name,"_time",(k-1),".png",sep="")
      png(image.name)
      image((-1)*geodesic[,,k])
      dev.off()
    }
  }
  if (create.gif==TRUE){
    if (!requireNamespace("animation", quietly = TRUE)) {
      stop("Package 'animation' required for creation of animated gif. Please install it or use option create.gif=FALSE")
    }
    gif.name<-paste(file.name,".gif",sep="")
    gif.generate(geodesic[,,2:(timesteps+1)],geodesic[,,1],geodesic[,,(timesteps+2)],timesteps,gif.name,delay.time,gif_type = "gif_im")
  }
  return(geodesic)
}
