#'Computes principal components of a data set of weighted point measures in R^d.
#'@description Takes a set of weighted point measures in R^d and their barycenter and computes their principal geodesics based on this.
#'@details This functions computes principal geodesics in the 2-Wasserstein space. It takes a set of discretized
#'or discrete input measures and computes iteratively principal directions using a gradient descent method. It
#'then returns the velocity fields describing the principal geodesics in the tangent space at the barycenter
#'of the data. This method requires the data to be supported on a rectangular grid in [0,1]. If one wants to
#'consider data with a more sparse or less regular support, it should be refered to the pca_ws or image_pca function.
#'@param data.weight A list of NxM matrices containing the weights of the data measures.
#'@param bary.weight A NxM matrix containing the weights of the barycenter of the data.
#'@param support A (NxM)x2 matrix containing the positions of the grid points. Make sure to align this
#'properly with the matrix positions of the weights.
#'@param sup_size A two-dimensional vector containing the value N and M.
#'@param t0.grid A tuning parameter shifting the center of the geodesic. Can remain at default. Might be set
#'to an vector containing a one dimensional grid centered around zero to obtain a slightly better approximation,
#'at the cost of a linear scaling in the number of grid points.
#'@param pca.count Integer specifying the number of principal geodesic to be computed.
#'@param maxiter Integer specifying the maximum number of iterations in the gradient descent.
#'@param submaxiter Integer specifying the maximum number of iterationns in the sub-algorithm.
#'@param tau.v Real value, specifying the step size in the gradient descent. This should be reduced, if one
#'encounters unreasonble results from the gradient descent.
#'@param tau.t Real value, specifying the step size in the sub algorithm. This should be reduced, if one
#'encounters unreasonble results from the gradient descent.
#'@return This function returns a list with two entries. The first entry is opt_times. This is a list containing
#'the optimal time points for each image for each principal geodesic respectively. This is for each pair of a data
#'measure and a geodesic the t in [0,1], for which the distance between the geodesic and the measure is minimal.\cr
#'The second entry is opt_velocity. This is a list containing a velocity field for each computed principal geodesic.
#'They describe the movement of the mass of the barycenter along each geodesic, respectively.
#'principal geodesic respectively. The positions of the k-th geodesic at time t in [-1,1] is given by
#'support+t * opt_velocity[[k]]. The weights are given by the weights of the barycenter at the corresponding
#'support points.
#'@examples grid<-WassersteinPCA:::build.grid(14,14)
#'res<-pca_grid(mnist_zero_mat,mnist_zero_bary_mat,grid,c(14,14))
#'@export
#'@references #' Cazelles, Elsa & Seguy, Vivien & Bigot, Jérémie & Cuturi, Marco & Papadakis, Nicolas; Log-PCA versus Geodesic PCA of histograms in the Wasserstein space;
#' SIAM Journal on Scientific Computing. 40; 2017.
pca_grid<-function(data.weight,bary.weight,support,sup_size,t0.grid=0,pca.count=1,maxiter=500,submaxiter=100,tau.t=10^-8,tau.v=10^-6){
  N<-sup_size[1]
  M<-sup_size[2]
  grid.size<-N*M
  hy<-abs(support[1,2]-support[2,2])
  hx<-abs(support[1,1]-support[(N+1),1])
  n<-length(data.weight)
  t.gpca<-vector("list",pca.count)
  t.gpca.opt<-vector("list",pca.count)
  sigma<-1/4
  theta<-tau.v/(1+2*tau.v)

  bary.wpp<-wpp(support,bary.weight)
  D<-diag(as.vector(bary.weight))
  data.wpp<-lapply(data.weight,wpp,coordinates=support)

  v_gpca<-vector("list",pca.count)
  v_gpca_opt<-vector("list",pca.count)
  v_gpca_opt_res<-vector("list",pca.count)
  threshold<-10^(-10)
  threshold_sub<-10^(-10)
  t0.count<-length(t0.grid)
  W_residual_g<-rep(Inf,pca.count)
  t0_opt<-vector("list",pca.count)
  ax<-min(support[,1])
  ay<-min(support[,2])
  bx<-max(support[,1])
  by<-max(support[,2])
  for (c in 1:pca.count){
    for (tt in t0.count){
      t0<-t0.grid[tt]
      #initialise
      xmin<-mapply(max,(ax-support[,1])/(t0+1),(bx-support[,1])/(t0-1),SIMPLIFY = FALSE)
      ymin<-mapply(max,(ay-support[,2])/(t0+1),(by-support[,2])/(t0-1),SIMPLIFY = FALSE)
      xmax<-mapply(min,(ax-support[,1])/(t0-1),(bx-support[,1])/(t0+1),SIMPLIFY = FALSE)
      ymax<-mapply(min,(ay-support[,2])/(t0-1),(by-support[,2])/(t0+1),SIMPLIFY = FALSE)
      mix<-sample(1:grid.size)
      v<-(support-support[mix,])/10
      t<-runif(n,-1,1)
      iter<-1
      err<-Inf
      WS_old<-Inf
      while((iter<=maxiter)&(err>threshold)){
        iter<-iter+1
        vml<-v
        tml<-t
        Z<-lapply(t,G_grid,t0=t0,sup=support,v=v)
        Z.wpp<-lapply(Z,wpp,bary.weight)
        P<-mapply(transport,data.wpp,Z.wpp,MoreArgs = list(p=2), SIMPLIFY = FALSE)
        WS<-mapply(wasserstein,data.wpp,Z.wpp,MoreArgs = list(p=2), SIMPLIFY = FALSE) #objective
        P<-lapply(P,tp.create,d1=grid.size,d2=grid.size)




        t<-mapply(gradient_update_t,t=t,P=P,Z=Z,MoreArgs = list(v=vml,t0=t0,tau=tau.t,d=D,bary=as.vector(bary.weight),X=support),SIMPLIFY = FALSE)
        v<-mapply(gradient_update_v,P=P,Z=Z,t=tml,MoreArgs = list(v=vml,t0=t0,tau=tau.v,d=D,X=support),SIMPLIFY=FALSE)
        v<-Reduce("+",v)
        WS<-Reduce("+",WS)
        z<-runif(grid.size,(-1/(t0+1)),(1/(1-t0)))
        v<-vml-tau.v*v
        u<-v
        ut<-v
        iter_sub<-1
        err_sub<-Inf
        while((iter_sub<=submaxiter)&(err_sub>threshold_sub)){
          iter_sub<-iter_sub+1
          u_old<-u
          z<-z+sigma*div_op(ut,N,M,hx,hy)
          z<-prox_s(z,sigma,t0)
          u<-u-theta*(div_opt(z,N,M,hx,hy)+((u-v)/tau.v))
          u<-prox_t(u,(c-1),v_gpca,as.vector(bary.weight))
          u.list<-lapply(seq_len(nrow(u)), function(i) u[i,])
          u<-t(mapply(proj_grid,u.list,xmin,xmax,ymin,ymax))
          ut<-2*u-u_old
          err_sub<-sum(abs(u_old-u))/sum(abs(u_old))
          if (is.infinite(err_sub)){
            err_sub<-sum(abs(u_old-u))
          }
          sub_obj<-sca(v-u,v-u)
        }
        v<-u
        t_old<-tml
        v_old<-vml
        err<-sum(abs(v_old-v))/sum(abs(v_old))
        # if (WS_old<=WS){
        #   print("Diverging")
        #   print(WS-WS_old)
        # }
        WS_old<-WS
      }
      t.gpca[[c]]<-unlist(t)+t0
      v_gpca[[c]]<-v
    }
    t.gpca.opt[[c]]<-t.gpca[[c]]
    v_gpca_opt[[c]]<-v_gpca[[c]]
  }
  return(list(opt_times=t.gpca.opt,opt_velocity=v_gpca_opt))
}
