#'Computes principal components of a data set of weighted point measures in R^2.
#'@description Computes principal components in the 2-Wasserstein Space
#'@details This function computes the principal components of a data set consisting of weighted point measures in R^2.
#'To do this it first maps the data to the tangent space at the barycenter and then performs standard euclidean PCA on this space.
#'Afterwards the resulting components are mapped back to the 2-Wasserstein Space.
#'@param image.weights A list of vectors containing the weights of the individial point measures/image pixels. All
#'vector entries are assumed to be greater 0.
#'@param image.pos A list of Nx2 matrices containing the positions of the data measures/pixels in [0,1]^2 in their respective rows.
#'@param bary.weights A vector specifying the weights of the barycenter. All vector entries are assumed to be greater 0.
#'@param bary.pos A Nx2 matrix specifying the positions of the barycenter.
#'@param pca.count An integer specifying the number of principal components to be computed.
#'@return The algorithm returns a list containing the entries components and evalues.
#'components contains a list of velocity fields describing the respective principal components.
#'evalues containts the eigenvalues corresponding to each principal component. This is also the variance of the data
#'in this specific direction.
#'The velocity fields paramatrizing the c-th principal component is then given by the product of the c-th component
#'and the square root of the c-th eigenvalue.
#'@export
#'@examples components<-pca_log(mnist_zero[[2]],mnist_zero[[1]],mnist_zero_bary[[2]],mnist_zero_bary[[1]])
#' for (c in 1:3){
#'  V<-components$components[[c]]*sqrt(components$evalues[c])
#'  name<-paste("RtestPCA","_pcanr_",c,sep="")
#'  visual_geodesic(mnist_zero_bary[[1]],mnist_zero_bary[[2]],V,V,timesteps = 100,
#'  grid.size=28*4,create.gif = FALSE,file.name=name)
#' }
#' @references Cazelles, Elsa & Seguy, Vivien & Bigot, Jérémie & Cuturi, Marco & Papadakis, Nicolas; Log-PCA versus Geodesic PCA of histograms in the Wasserstein space; SIAM Journal on Scientific Computing. 40; 2017. \cr
#' A Linear Optimal Transportation Framework for Quantifying and Visualizing Variations in Sets of Images; International Journal of Computer Vision;
#' Wang, Wei & Slepcev, Dejan & Basu, Saurav & Ozolek, John A. & Rohde, Gustavo K.;2013.
pca_log<-function(image.weights,image.pos,bary.weights,bary.pos,pca.count=3){
  N<-length(image.weights)
  image.sizes<-lapply(image.weights,length)
  bary.size<-length(bary.weights)
  bary.wpp<-wpp(bary.pos,bary.weights)
  image.wpp<-mapply(wpp,image.pos,image.weights,SIMPLIFY=FALSE)
  P<-lapply(image.wpp,transport,a=bary.wpp,p=2)
  P<-mapply(tp.create,P,image.sizes,MoreArgs = list(d1=bary.size),SIMPLIFY=FALSE)
  V<-mapply(bary_proj,image.pos,P,MoreArgs = list(bary=bary.weights),SIMPLIFY=FALSE)
  V<-lapply(V,as.vector)
  V.mat<-t(matrix(unlist(V),2*length(bary.weights),N))
  V.bar<-Reduce('+',V)/N
  tmp<-lapply(V,cov_mat,mean=V.bar)
  S<-Reduce('+',tmp)/N
  decomp<-eigen(S)
  EigV<-decomp$vectors/sqrt(bary.weights)
  tV<-t(t(V.mat)-V.bar)%*%diag(c(bary.weights,bary.weights))%*%EigV
  PC<-vector("list",pca.count)
  for (c in 1:pca.count){
    TV<-EigV[,c]
    PC[[c]]<-matrix(TV,bary.size,2)
  }
  return(list(components=PC,evalues=decomp$values))
}
