#'Computes principal components of a data set of weighted point measures in R^d.
#'@description Takes a set of weighted point measures in R^d and their barycenter and computes their principal geodesics based on this.
#'@details Let Y be the positions of the barycenter in R^d, then each principal geodesic can be parametrized as G(t)=Y-V1+t(V1+V2) for t in [0,1] and two
#'velocity fields V1 and V2. This corresponds to the definition of a generalized geodesic. If one wishes to obtain a true geodesic, one can use the regularization
#'parameter lambda to force the fields to be collinear, but for visual analysis this is often not neccesary. This function takes data sets consisting of sums of
#'weighted point measures and computes V1 and V2 via projected gradient descent. For each pca that is to be computed the corresponding V1 and V2 are returned.
#'Since we are restricted to point measures, the velocity fields are simple Nxd matrices here, where each row corresponds to a vector in R^d along which the mass
#'is transported on the geodesic.\cr
#'This function is mostly identical to the \link{pca_ws} function. The only difference is, that the former uses an entropy
#'regularization to approximate the Wasserstein-Distances, while this function explicitly solves the corresponding to optimal
#'transport problem by calling the transport-package. The pca_ws_tp function is therefore slightly slower, but also
#'a little more precise.
#'@param data.pos A list of Nxd matrices of data positions.
#'@param data.weights A list of N-dim vectors of the data weights.
#'@param bary.pos A Nxd matrix of the positions of the barycenter
#'@param bary.weights A N-dim vector of the weights of the barycenter
#'@param stepsize The step size of the gradient descent algorithm. Note choosing this too high might lead to divergence, but choosing it too small might lead to
#'long runtimes.
#'@param lambda This is a regularisation parameter which is greater or equal to zero. Choosing this greater than zero will lead to a regularization step,
#'where the velocity fields are forced to be collinear.
#'@param pca.count Integer determining the amount of principal components to be computed.
#'@param K Integer determining the size of the time grid in the computation. The defauls usually works fine, but in case of bad convergence properties it might
#'be helpful to increase this paramter. Note that the run time is linear in K.
#'@param max_iter Integer specifying the maximal amount of iterations in the gradient descent method.
#'@param error The value of change between iterations at which the gradient descent should terminate.
#'@return The function returns an array of the size pca.countx2xNxd, where result[k,1,,] contains the first velocity field of the k-th pca and result[k,2,,] contains
#'the second velocity field of the k-th pca.
#'@export
#'@examples res<-pca_ws_tp(mnist_zero[[1]],mnist_zero[[2]],mnist_zero_bary[[1]],mnist_zero_bary[[2]],
#'10^(-4),0,1,20,400,10^(-6))
#' visual_geodesic(mnist_zero_bary[[1]],mnist_zero_bary[[2]],res[1,1,,],res[1,2,,])
#' @references Seguy, Vivien and Cuturi, Marco; Principal Geodesic Analysis for Probability Measures under the Optimal Transport Metric; Advances in Neural Information Processing Systems 28; 2015.
pca_ws_tp<-function(data.pos,data.weights,bary.pos,bary.weights,stepsize,lambda,pca.count,K=20,max_iter,error){
  bary.length<-dim(bary.pos)[1]
  data.length<-length(data.pos)
  N<-data.length
  p<-dim(bary.pos)[2]
  bary.diag<-diag((bary.weights)^(-1))
  pca.out<-array(0,c(pca.count,2,bary.length,p))
  data.wpp<-mapply(wpp,data.pos,data.weights,SIMPLIFY = FALSE)
  data.size<-lapply(data.weights,length)
  update_step<-function(G,B,P,D){
    out<-G-(B%*%t(P)%*%D)
    return(out)
  }
  for (n in 1:pca.count){
    V1<-cbind(runif(bary.length,-1,1),runif(bary.length,-1,1))/100
    V2<-cbind(runif(bary.length,-1,1),runif(bary.length,-1,1))/100
    objective_new<-Inf
    objective_old<-Inf
    count<-0
    while ( ((count == 0) || (objective_old - objective_new > error) || (objective_old - objective_new < 0)) && (count < max_iter)){
      count<-count+1
      objective_old<-objective_new
      #Compute the optimal projection time t_i* for each data measure
      time.grid<-(0:K)/K
      grid.search<-function(d.pos,g.pos,d.weight,g.weight){
        d.wpp<-wpp(d.pos,d.weight)
        g.wpp<-lapply(g.pos,wpp,mass=g.weight)
        out<-lapply(g.wpp,wasserstein,a=d.wpp,p=2)
        out<-unlist(out)
        return(out)
      }
      Geodesic<-lapply(time.grid,G,bary.pos,V1,V2)
      tmp<-mapply(grid.search,d.pos=data.pos,d.weight=data.weights,MoreArgs=list(g.pos=Geodesic,g.weight=bary.weights))
      ws.dist<-t(tmp)

      pick_time<-function(times,K){
        out<-(which.min(times)-1)/K
        return(out)
      }
      optimal.times<-apply(ws.dist,1,pick_time,K=K)
      objective_new<-apply(ws.dist,1,min)
      objective_new<-sum(objective_new)
      Geodesic<-lapply(optimal.times,G,bary.pos,V1,V2)
      geo.wpp<-lapply(Geodesic,wpp,mass=bary.weights)
      tplan<-mapply(transport,data.wpp,geo.wpp,MoreArgs = list(p=2),SIMPLIFY=FALSE)
      tplan<-mapply(tp.create,tplan,data.size,MoreArgs=list(d2=bary.length),SIMPLIFY=FALSE)
      temp<-mapply(update_step,G=Geodesic,P=tplan,D=data.pos,MoreArgs = list(B=bary.diag),SIMPLIFY=FALSE)
      opt1<-2*(1-optimal.times)
      opt2<-2*optimal.times
      temp.grad1<-mapply('*',temp,opt1,SIMPLIFY = FALSE)
      temp.grad2<-mapply('*',temp,opt2,SIMPLIFY = FALSE)
      grad.v1<-Reduce('+',temp.grad1)
      grad.v2<-Reduce('+',temp.grad2)

      if (lambda>0){
        norm.V1<-sqrt(sca_l2(V1,V1,bary.weights))
        norm.V2<-sqrt(sca_l2(V2,V2,bary.weights))
        pen.grad.v1<-2*sqrt(h(V1,V2,bary.weights))*(V2-(V1*norm.V2/norm.V1))
        pen.grad.v2<-2*sqrt(h(V1,V2,bary.weights))*(V1-(V2*norm.V1/norm.V2))
        grad.v1<-grad.v1+lambda*pen.grad.v1
        grad.v2<-grad.v2+lambda*pen.grad.v2
      }
      V1<-V1-stepsize*grad.v1
      V2<-V2-stepsize*grad.v2
      if (n>1){
        for (k in 1:(n-1)){
          V<-pca.out[k,1,,]+pca.out[k,2,,]
          V1<-V1-((sca_l2(V,V1,bary.weights)/sca_l2(V,V,bary.weights))*V)
          V2<-V2-((sca_l2(V,V2,bary.weights)/sca_l2(V,V,bary.weights))*V)
        }
      }
      bary.v1<-bary.pos-V1
      grid.wpp<-wpp(bary.pos,bary.weights)
      V1.wpp<-wpp(bary.pos-V1,bary.weights)
      tplan<-transport(grid.wpp,V1.wpp,p=2)
      tplan<-tp.create(tplan,bary.length,bary.length)
      V1_proj<-(bary.diag%*%t(tplan)%*%bary.v1)
      V1<-(-1)*(V1_proj-bary.pos)
      bary.v2<-bary.pos+V2
      grid.wpp<-wpp(bary.pos,bary.weights)
      V2.wpp<-wpp(bary.pos+V2,bary.weights)
      tplan<-transport(grid.wpp,V2.wpp,p=2)
      tplan<-tp.create(tplan,bary.length,bary.length)
      V2_proj<-(bary.diag%*%t(tplan)%*%bary.v2)
      V2<-V2_proj-bary.pos
    }
    pca.out[n,1,,]<-V1
    pca.out[n,2,,]<-V2
  }
  return(pca.out)
}
